/* $Id$
 *
 * dChecker - Distributed Behavior Protocols Checker 
 * Copyright (C) 2006-2007  DSRG, Charles University in Prague
 *                          http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */

package org.ow2.dsrg.fm.dchecker;

import org.ow2.dsrg.fm.dchecker.rmi.RMIClientImpl;
import org.ow2.dsrg.fm.bptools.util.Log;






/**
 * Main class of Distributed Behaviour Protocol Checker Client.
 *
 * The client purpose is to traverse parts of state spaces of protocols.
 * The clients are controlled by server.
 * @author thorm
 */
public class DCheckerClient {
    
    public static final int DEFAULTPORT = 5527;
    
    private static void printHelp() {
        System.out.println("Usage: client serverAddress [port for listenning]");
    }
    
    /**
     * Run the client on this machine
     * The first argument is address of the server that will controll this client
     * @param args list of arguments
     * @throws java.lang.Exception thrown if something goes wrong.
     */
    public static void main(String[] args) throws Exception {
        Log.init();
        
        if (args.length != 1 && args.length != 2 || args[0].equals("--help")) {
            printHelp();
            System.exit(1);
        }
        
        int listenningPort = DEFAULTPORT;
        if (args.length == 2){
            try{
                listenningPort = Integer.parseInt(args[1]);
            }catch (NumberFormatException nfe){
                printHelp();
                System.exit(1);
            }
        }
        
        RMIClientImpl i = new RMIClientImpl(args[0], listenningPort);

        i.run();    
    }
}
