/* $Id$
 *
 * dChecker - Distributed Behavior Protocols Checker 
 * Copyright (C) 2006-2007  DSRG, Charles University in Prague
 *                          http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */

package org.ow2.dsrg.fm.dchecker;


/**
 * Used to run all components within one virtual machine - for debugging purposes.
 * @author thorm
 */
public class DistributedTest {
    
    
    static class ClientRunnable implements Runnable{
        
         public ClientRunnable(int p){
             port = p;
         }
         
         int port; 
         public void run() {
                try {
                    DCheckerClient.main(new String[] { "localhost", String.valueOf(port) });
                } catch (Exception e) {
                    e.printStackTrace();
                }
         }
    }
    
    
    private static void printHelp() {
        System.out.println("Usage: dCheckerDTest file.bp [number of threads] ");
    }
    
    
    /**
     * Run client, server and console in one virtual machine
     * @param args list of arguments ( not used) 
     * @throws java.lang.Exception error reporting
     */
    public static void main(String args[]) throws Exception {
        
        int clients = 1;
        
        if (args.length != 1 && args.length != 2 || args[0].equals("--help")) {
            printHelp();
            System.exit(1);
        }
    
        if (args.length == 2){
            try{
                clients = Integer.parseInt(args[1]);
            }catch (NumberFormatException nfe){
                printHelp();
                System.exit(1);
            }
        }
        
        final String fileName = args[0];
                
        Runnable r1 = new Runnable() {
            public void run() {
                try {
                    DCheckerServer.main(new String[] {});
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        };
        
        
        
        Runnable r3 = new Runnable() {
            public void run() {
                try {
                    DCheckerConsole.main(new String[]{"localhost",fileName});
                } catch (Exception e) {
                    e.printStackTrace();
                }

            }
        };
        
        new Thread(r1,"Server").start();
        Thread.sleep(5000);
        
        
        int clientPort = 5527;
        
        for (int i=0;i<clients;i++){
            new Thread(new ClientRunnable(clientPort++), "Client"+i).start();
            Thread.sleep(2000);
        }
        
        r3.run();
        
        System.exit(0);
        
    }
    
    
}
