/* $Id$
 *
 * dChecker - Distributed Behavior Protocols Checker 
 * Copyright (C) 2006-2007  DSRG, Charles University in Prague
 *                          http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */

package org.ow2.dsrg.fm.dchecker.distributed;


import org.ow2.dsrg.fm.dchecker.distributed.spacemap.BoundingBox;
import org.ow2.dsrg.fm.dchecker.errors.CompositionException;
import org.ow2.dsrg.fm.dchecker.fly.FlyFSM;
import org.ow2.dsrg.fm.dchecker.fly.FlyState;
import org.ow2.dsrg.fm.dchecker.rmi.MachineLostException;
import org.ow2.dsrg.fm.dchecker.rmi.RMIClientTaskImpl;
import java.util.LinkedList;


/**
 *
 * @author pocht1am
 */
public class BFSArea extends Area {
    
    /**
     * Constructor
     *
     * The area position is given by the bounding box
     * @param si position and size of new area
     */
    public BFSArea(BoundingBox si) {
        this(si.getMinimalCoord(), si.getMaximalCoord());
        
    }
    
    /**
     * The new area position and size is given by two corners
     * @param min coordinates of first corner
     * @param max coordinates of second corner
     */
    public BFSArea(int[] min, int[] max) {
        super(min, max);
        splittedState = new int[min.length];
        splittedNewState = new int[min.length];
        visitedLocal = new VisitedNodes(toString()+" Visited: ");
        visitedOther = new VisitedNodes(toString()+" Outgoing: ");
    }
    
    {
        queue = new LinkedList<FlyState>();
    }
    
    
    
    public void newEntryPoint(FlyState s) {
        if (visitedLocal.insertRoot(s)) {
            queue.add(s);
            if (userHandler!=null){
                ffsm.getStateParts(s, splittedState);
                userHandler.node(splittedState);
                assert belongs(splittedState);
            }
        }
    }
    
    
    public boolean traversal(RMIClientTaskImpl task) throws CompositionException, MachineLostException{
        
        FlyState first;
        if ((first = queue.poll())!=null) {
            FlyState child;
            FlyFSM.StateNeighbourEnumeration neighEnum = ffsm.getNeighboursEnum(first);
            
            if (userHandler!=null){
                    ffsm.getStateParts(first, splittedState);
                }
            
            /*
            System.out.println("Node");
            ffsm.getStateParts(first, splittedNewState);
            assert(belongs(splittedNewState));
            */
            
            while (neighEnum.hasMoreElements()) {
                
                child = neighEnum.nextElement();
                
                // even the states out of the area are inserted into visited nodes ( to get the parent )
                
                ffsm.getStateParts(child, splittedNewState);
                
                if (belongs(splittedNewState)) {
                    if (visitedLocal.insert(child, first)) {
                        queue.add(child);
                        if (userHandler!=null){
                            userHandler.node(splittedNewState);
                            userHandler.spanningEdge(splittedState,splittedNewState);
                        }
                    } else{
                        if (userHandler!=null){
                            userHandler.edge(splittedState,splittedNewState);
                        }
                    }
                } else {
                    synchronized (this){
                        if (visitedOther.insert(child, first)) {
                            task.outOfArea(child, splittedNewState);
                            if (userHandler!=null){
                                //userHandler.node(splittedNewState);
                                userHandler.spanningEdge(splittedState,splittedNewState);
                            }
                        } else{
                            if (userHandler!=null){
                                userHandler.edge(splittedState,splittedNewState);
                            }
                        }
                    }
                }
            }
            return true;
        }else{
            return false;
        }
        
        
    }
    
    public Area getSubtask() {
        assert false;
        return null;
    }
    
    protected LinkedList<FlyState> queue;
    
    
}
