/* $Id$
 *
 * dChecker - Distributed Behavior Protocols Checker 
 * Copyright (C) 2006-2007  DSRG, Charles University in Prague
 *                          http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */

package org.ow2.dsrg.fm.dchecker.distributed.spacemap;


import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;


/**
 * Represents the function from state space of cartesian product of automatons to
 * T
 * The state space is divided into areas that has assigned come value of type T
 * 
 * The number of automatons can be from 1 to n
 * @author Tomas Poch
 */
public class DistributionInfo<T> implements SpaceMap<T> {
    
    /**
     * Creates a new instance of DistributionInfo
     * From the given parameter decides to wrap NDimSpaceMap or LeafSpaceMap(dim = 1)
     * @param dimension dimension of state space<br>
     * Number of protocols in the architecture
     */
    public DistributionInfo(int dimension) {
        assert dimension != 0;
        if (dimension == 1) {
            data = new LeafSpaceMap();
        } else {
            data = new NDimSpaceMap(dimension);
        }
        
        // volumes = new HashMap<T, Long>();
    }
    
    /**
     * Assign value to the part of state space
     * @param keyRange range the value is assigned to
     * @param value assigned value
     */
    public void addInterval(BoundingBox keyRange, T value) {

        /*
         T oldValue = data.get(keyRange.getMinimalCoord());
         
         
         long volumeOfOldValue = 0;
         long volumeOfNewValue = 0;
         
         if (oldValue != null) {
         if (oldValue.equals(value)) {
         return;
         }
         volumeOfOldValue = volumes.get(oldValue);
         Long tmp = volumes.get(value);

         volumeOfNewValue = (tmp == null) ? 0 : volumes.get(value);
         }
         
         long d = keyRange.getVolume();

         volumeOfNewValue += d;
         if (volumeOfOldValue != 0) {
         volumeOfOldValue -= d;
         volumes.put(oldValue, volumeOfOldValue);
         }
         volumes.put(value, volumeOfNewValue);
         assert volumeOfOldValue >= 0;
         */
        
        data.addInterval(keyRange, value);
        
    }
    
    /**
     * Return list of values from T assigned to some subspace. The list is sorted
     * by the volume of areas that are assigned to the values from the list.
     * @return list of distinct values
     */
    
    /*
     public List<T> sortByVolume() {
     ArrayList<VolumeOrderedItem<T>> ret = new ArrayList<VolumeOrderedItem<T>>();

     for (T key:volumes.keySet()) {
     ret.add(new VolumeOrderedItem<T>(key, volumes.get(key)));
     }
     Collections.sort(ret);
     List<T> ret2 = new LinkedList<T>();

     for (int i = 0; i < ret.size(); i++) {
     ret2.add(ret.get(i).value);
     }
     return ret2;
     }
     */
    
    /**
     * Get value assigned to the given coordinates
     * @param key coordinates
     * @return assigned value
     */
    public T get(int[] key) {
        return data.get(key);
    }


    
    public boolean changeValueForWholeArea(int[] key,T value){
        return data.changeValueForWholeArea(key,value);
    }
    
    /**
     * Print the content of this structure. For debugging purposes
     * @return string representation of this structure
     */
    public String toString() {
        ArrayList<String> al = new ArrayList<String>();
        LinkedList<String> ll = new LinkedList<String>();
        
        data.toStringList(al, ll);
        Collections.sort(al);
        
        StringBuffer sb = new StringBuffer();

        for (String s:al) {
            sb.append(s);
            sb.append("\n");
        }
        return sb.toString();
    }
    
    /**
     * keeps the data
     */
    SpaceMapImpl<T> data;
    
    /**
     * keeps computed volumes
     */
    // HashMap<T, Long> volumes;
    
    /**
     * Used to sort items by volume
     
     static class VolumeOrderedItem<T> implements Comparable<VolumeOrderedItem<T>> {
     
     
     * Constructor
     * @param value assigned to given volume
     * @param volume to be compared with other volumes
     
     public VolumeOrderedItem(T value, long volume) {
     this.value = value;
     this.volume = volume;
     }
     
     /**
     * Value whose position is determined by its volume
     
     T  value;

     /**
     * volume of value
     
     long volume;
     
     /**
     * used by java sorting algorithms
     * @param second value to be compared with
     * @return -1 if second value is bigger, 0 if second value is equal and 1 otherwise
     
     public int compareTo(VolumeOrderedItem<T> second) {
     return (int) (volume - second.volume);
     }
     }
     */
}
