/* $Id$
 *
 * dChecker - Distributed Behavior Protocols Checker 
 * Copyright (C) 2006-2007  DSRG, Charles University in Prague
 *                          http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */

package org.ow2.dsrg.fm.dchecker.distributed.spacemap;

import java.io.Serializable;


/**
 * Interval with redefined comparison to be able to store these intervals in trees
 * 
 * The interval contains its beginning and does not contain its end.
 * &lt;left,right)
 * @author Tomas Poch
 */
public class Range<T> implements Comparable<Range<T>>,Serializable {

    /**
     * Create new Range instance
     * @param l begin of interval
     * @param r end of interval
     */
    public Range(int l, int r) {
        update(l, r);
    }
    
    /**
     * Change interval
     * @param l new begin of interval
     * @param r new end of interval
     */
    public void update(int l, int r) {
        left = l;
        right = r;
    }
    
    /**
     * Change interval to contain only one number
     * @param i new value
     */
    public void update(int i) {
        left = i;
        right = i + 1;
    }
    
    /**
     * Used by trees to sort the values
     * Interval a &lt; interval b &lt;=&gt; a.right &lt; b.left
     * @param o to be compared
     * @return comparison result
     */
    public int compareTo(Range<T> o) {
        if (o.right <= left) {
            return 1;
        }
        if (o.left >= right) {
            return -1;
        }
        return 0;
    }
    
    /**
     * Return true of the r is subset of this range
     * @param r other range
     * @return true of the r is subset of this range
     */
    public boolean isParameterSubrange(Range<T> r) {
        return (r.left >= left && r.right <= right);
    }
    
    /**
     * Return true if the given range does not intersect this range.
     * @param r other range
     * @return true if the given range does not intersect this range.
     */
    public boolean noIntersection(Range<T> r) {
        return (r.left >= right) || (r.right <= left);
    }
    
    /**
     * begin of the interval
     */
    int left;

    /**
     * end of the interval
     */
    int right;

    /**
     * value assigned to the interval
     */
    T value;
}
