/* $Id$
 *
 * dChecker - Distributed Behavior Protocols Checker 
 * Copyright (C) 2006-2007  DSRG, Charles University in Prague
 *                          http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */

package org.ow2.dsrg.fm.dchecker.fly;


/**
 *
 * @author thorm
 *
 */
public class BigInt implements java.io.Serializable {
    
    public static final long intMask = 0xffffffff;
    
    public BigInt() {}
    
    public BigInt(BigInt oldState) {
        data = oldState.data.clone();
    }

    /**
     * State identifier
     */
    public int[] data;

    

    
    public void resize(int bitWidth) {
        
        int arraySize;
        if (bitWidth%32==0){
            arraySize = (bitWidth / 32);
        }else{
            arraySize = (bitWidth / 32)+1;
            
        }

        if (data == null) {
            data = new int[arraySize];
        } else if (arraySize > data.length) {
            int[] newData = new int[arraySize];

            for (int i = 0; i < data.length; i++) {
                newData[i] = data[i];
            }
            data = newData;
        }
    }
    
    public long getUnsignedInt(int index) {
        long l = data[index];

        if (l < 0) {
            l = (l & 0xffffffffL) | 0x80000000L;
        }
        assert l >= 0;
        return l;
    }
    
    public void setPart(int part, BitPartInfo partInfo) {
        
        int index = partInfo.firstBitPosition / 32;
        long l = getUnsignedInt(index);

        if (index + 1 < data.length) {
            l += getUnsignedInt(index + 1) << 32;
        }    

        l = l & (~partInfo.mask);
        long p = part;

        p = p << partInfo.firstBitPosition % 32;
        l = l | p;
        
        data[index] = (int) (l & intMask);
        if (index + 1 < data.length) {
            data[index + 1] = (int) (l >>> 32);
        }    

    }
    
    public int getPart(BitPartInfo partInfo) {
        int index = partInfo.firstBitPosition / 32;
        long l = getUnsignedInt(index);

        if (index + 1 < data.length) {
            l += getUnsignedInt(index + 1) << 32;
        }    
        
        l = (l & partInfo.mask) >>> partInfo.firstBitPosition % 32;
        assert ( l < Integer.MAX_VALUE);
        return (int) (l);
        
    }
    
    @Override
    public boolean equals(Object o) {
        BigInt bi = (BigInt) o;

        for (int i = 0; i < data.length; i++) {
            if (data[i] != bi.data[i]) {
                return false;
            }
        }
        return true;
    }
    
    @Override
    public int hashCode() {
        int ret = 0;

        for (int i = 0; i < data.length; i++) {
            ret += data[i];
        }
        return ret;
        
    }

}
