/* $Id$
 *
 * dChecker - Distributed Behavior Protocols Checker 
 * Copyright (C) 2006-2007  DSRG, Charles University in Prague
 *                          http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */

package org.ow2.dsrg.fm.dchecker.fly;


import java.util.ArrayList;
import java.util.Collections;
import java.util.Collection;
import java.util.Iterator;
import org.ow2.dsrg.fm.bptools.fsm.SimpleEdge;
import org.ow2.dsrg.fm.bptools.fsm.SimpleState;
import org.ow2.dsrg.fm.bptools.fsm.State;


/**
 *
 * @author thorm
 */
public class FastSimpleState extends SimpleState {
    
    /** Creates a new instance of FastSimpleState */
    public FastSimpleState(boolean isFinal, Collection<SimpleEdge> edgesLst) {
        super(isFinal);
        
        ArrayList<SimpleEdge> al = new ArrayList<SimpleEdge>(edgesLst.size());
        al.addAll(edgesLst);
        Collections.sort(al/*, new Comparator<SimpleEdge>() {
            public int compare(SimpleEdge a, SimpleEdge b) {
                return a.label - b.label;
            }
        }*/);
        
        if (al.size() > 0) {
            base = al.get(0).getLabel();
            int size = (al.get(al.size() - 1).getLabel()) - base + 1;
            
            edges = new int[size];
            
            Iterator<SimpleEdge> it = al.iterator();
            int nextStop;
            int i = 0;

            while (it.hasNext()) {
                SimpleEdge e = it.next();

                nextStop = e.getLabel() - base;
                for (; i < nextStop; i++) {
                    edges[i] = -nextStop;
                }
                edges[nextStop] = e.getTarget();
                i++;
            }
            
        }
        
    }
    
    public int getEdgeTarget(int label) {
        if (edges == null) {
            return -1;
        }
        int idx = label - base;

        if (0 <= idx && idx < edges.length) {
            return edges[idx];
        }
        return -1;
    }
    
    public int getLabelForTarget(int target) {
        for (int i = 0; i < edges.length; i++) {
            if (edges[i] == target) {
                return i + base;
            }
        }

        return State.NO_SUCH_EDGE;
    }
    
    public int nextEdge(int i) {
        
        if (edges == null) {
            return State.AFTER_LAST_EDGE;
        }
        int idx = i - base + 1;

        if (idx <= 0) {
            return base;
        }
        if (idx >= edges.length) {
            return State.AFTER_LAST_EDGE;
        }
        int ret = edges[idx];
        
        if (ret < 0) {
            return (-ret) + base;
        }
        return i + 1;
        
    }
    
    /**
     * Array of edges. The label is determined by the position. Size is equal
     * to the number of events in the architecture
     */
    int base;
    int[] edges;
}
