/* $Id$
 *
 * dChecker - Distributed Behavior Protocols Checker 
 * Copyright (C) 2006-2007  DSRG, Charles University in Prague
 *                          http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */

package org.ow2.dsrg.fm.dchecker.fly;

import java.io.PrintStream;
import java.io.Serializable;
import java.util.Set;

import org.ow2.dsrg.fm.dchecker.distributed.spacemap.BoundingBox;
import org.ow2.dsrg.fm.dchecker.errors.CompositionException;
import org.ow2.dsrg.fm.bptools.IEventTable;


/**
 *
 * @author thorm
 */
public abstract class FlyFSM implements Serializable {
    
    /** Creates a new instance of AbstractFlyFSM */
    public FlyFSM(Set<Integer> u) {
        unbound = u;
    }

    
    public abstract StateNeighbourEnumeration getNeighboursEnum(FlyState fs);

    /**
     * Fill the given array with identifiers of actual states of the automatons
     * the consent operator consist from
     * 
     * @param state state identifier that consist of identifiers of all automatons
     * @param parts array of the correct size used to store the result
     */
    public abstract void getStateParts(FlyState state, int[] parts);

    protected Set<Integer> unbound;
    
    public abstract class StateNeighbourEnumeration implements Cloneable {
        
        public StateNeighbourEnumeration(FlyState fs) {
            f = fs;
            
        }
        
        public Object clone() {
            try {
                return super.clone();
            } catch (CloneNotSupportedException e) {
                assert false;
            }
            return null;
        }
        
        public FlyState getState() {
            return f;
        }

        public abstract FlyState nextElement() throws CompositionException;
        
        public abstract boolean hasMoreElements() throws CompositionException;
        
        FlyState f;
        
    }

    /**
     * Set the size of the given bounding box to contain whole state space
     * @param bb to be filled
     */
    public abstract void fillBoundingBox(BoundingBox bb);

    public abstract int[][] getEdgeDensity();
    
    /**
     * Set the initial state.
     * @param state store the initial state here.
     */
    public abstract void init(FlyState state);

    /**
     * Get identifier size in bits
     * @return identifier size in bits
     */
    public abstract int identifierSize();

    
    /**
     * Return the number of automatons this automaton consist from
     * @return number of automatons
     */
    public abstract int getDimension();

    /**
     * Return true if the given state is final.
     * @param s state
     * @return true iff the given state is final
     */
    public abstract boolean isFinal(FlyState s);

    public abstract void dump(PrintStream s, IEventTable et);
    
    protected static String eventString(IEventTable et, int e){
        return (et==null)? Integer.toString(e) : et.getEventString(e)+"(" + e + ")";
    }
    
    protected static void dumpEventList(PrintStream os, IEventTable et, Set<Integer> events){
        for (Integer e:events){
                String labelStr = eventString(et, e);
                os.println("    "+labelStr+", ");
            }
    }
}
