/* $Id$
 *
 * dChecker - Distributed Behavior Protocols Checker 
 * Copyright (C) 2006-2007  DSRG, Charles University in Prague
 *                          http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */

package org.ow2.dsrg.fm.dchecker.fly;



import java.io.PrintStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.ow2.dsrg.fm.bptools.BPEventTableImpl;
import org.ow2.dsrg.fm.bptools.IEventTable;
import org.ow2.dsrg.fm.bptools.fsm.State;
import org.ow2.dsrg.fm.dchecker.distributed.spacemap.BoundingBox;
import org.ow2.dsrg.fm.dchecker.errors.BadActivityException;
import org.ow2.dsrg.fm.dchecker.errors.CompositionException;
import org.ow2.dsrg.fm.dchecker.errors.NoActivityException;


/**
 *
 * @author thorm
 */
public class FlyMultiConsent extends FlyFSM {
    
    /** Creates a new instance of FlyMultiConsent */
    public FlyMultiConsent(
            SimpleFlyFSM[] l,
            ArrayList<Set<Integer>> bindings,
            Set<Integer> unbound
            ) {
        super(unbound);
        fsms = l;
        int dim = fsms.length;

        eventToFsm = new HashMap<Integer, Integer>();
        
        BitPartInfo base = BitPartInfo.ZERO;

        for (int i = 0; i < dim; i++) {
            SimpleFlyFSM f = fsms[i];

            f.partIndex = i;
            f.setBase(base);
            base = base.multiply(f.getMaxState());
        }
        
        for (int i = 0; i < dim - 1; i++) {
            for (Integer method: bindings.get(i)) {
                eventToFsm.put(method, i);
                
                // ////
                // System.out.println(method + "->"+ i);
                // ////
            }
        }
    }
    
    public void fillBoundingBox(BoundingBox bb) {
        for (SimpleFlyFSM f:fsms) {
            f.fillBoundingBox(bb);
        }
    }
    
    public void init(FlyState state) {
        for (SimpleFlyFSM f:fsms) {
            f.init(state);
        }
    }
    
    
    public int identifierSize(){
        return fsms[fsms.length-1].identifierSize();
    }
    
    public void getStateParts(FlyState state, int[] parts) {
        for (int i = 0; i < fsms.length; i++) {
            
            SimpleFlyFSM f = fsms[i];

            parts[i] = f.getStatePart(state);
        }
    }
    
    public boolean isFinal(FlyState s) {
        for (SimpleFlyFSM f:fsms) {
            if (!f.isFinal(s)) {
                return false;
            }
        }
        return true;
    }
    
    public StateNeighbourEnumeration getNeighboursEnum(FlyState fs) {
        return new StateNeighbourEnumeration(fs) {
            public boolean hasMoreElements() throws CompositionException {
                
                while (true) {
                    nextEdgeLabel = fsms[automatonIndex].getNextSymbol(f,
                            nextEdgeLabel);
                    if (nextEdgeLabel == State.AFTER_LAST_EDGE) {
                        automatonIndex++;
                        if (automatonIndex == fsms.length) {
                            if (!emitActionsOnTheLeft.isEmpty()) {
                                Iterator<Integer> it = emitActionsOnTheLeft.iterator();
                                it.hasNext();
                                throw new BadActivityException(f, it.next());
                            }
                            if (firsthasMoreElementsInvocation && !isFinal(f)) {
                                throw new NoActivityException(f);
                            }
                            return false;
                        }
                        nextEdgeLabel = State.BEFORE_FIRST_EDGE;
                        continue;
                    }

                    bindingFollowsThisAutomaton = eventToFsm.get(nextEdgeLabel);
                    int bfta;

                    if (bindingFollowsThisAutomaton != null) {
                        bfta = bindingFollowsThisAutomaton;
                        if (bfta == automatonIndex) {
                            if (BPEventTableImpl.staticIsEmit(nextEdgeLabel)) {
                                emitActionsOnTheLeft.add(nextEdgeLabel);
                            }
                            continue;
                        }
                        if (bfta < automatonIndex
                                && BPEventTableImpl.staticIsAccept(nextEdgeLabel)) {
                            int complement = BPEventTableImpl.getComplement(
                                    nextEdgeLabel);

                            if (!fsms[bfta].accepts(f, complement)) {
                                continue;
                            }
                            emitActionsOnTheLeft.remove(complement);
                        }
                    } else if (unbound.contains(nextEdgeLabel)){
                        if (BPEventTableImpl.staticIsEmit(nextEdgeLabel)) {
                            throw new BadActivityException(f, nextEdgeLabel);
                        }else{
                            continue;
                        }
                    }
                            
                    
                    firsthasMoreElementsInvocation = false;
                    return true;
                }
                
            }
            
            public FlyState nextElement() throws CompositionException {
                FlyState ret = new FlyState(f);

                if (bindingFollowsThisAutomaton != null
                        && bindingFollowsThisAutomaton < automatonIndex) {
                    
                    int complement = BPEventTableImpl.getComplement(nextEdgeLabel);

                    if (!fsms[bindingFollowsThisAutomaton].follow(f, ret,
                            complement)) {
                        throw new BadActivityException(f, nextEdgeLabel);
                    }
                    
                }
                
                boolean followOk = fsms[automatonIndex].follow(f, ret,
                        nextEdgeLabel);

                assert followOk;
                
                return ret;
                
            }
            
            HashSet<Integer> emitActionsOnTheLeft = new HashSet<Integer>();
            Integer bindingFollowsThisAutomaton;
            boolean firsthasMoreElementsInvocation = true;
            int automatonIndex;
            int nextEdgeLabel = State.BEFORE_FIRST_EDGE;
        };
    }
    
    
    public int[][] getEdgeDensity(){
        int[][] ret = new int[getDimension()][];
        for(int i=0;i<ret.length;i++){
            ret[i] = fsms[i].getEdgeDensity()[0];
        }
        
        return ret;
        
    }
    
    SimpleFlyFSM[] fsms;
    HashMap<Integer, Integer> eventToFsm;

    /**
     * Return the number of automatons this automaton consist from
     * 
     * @return number of automatons
     */
    public int getDimension() {
        return fsms.length;
    }
    

    
    public void dump(PrintStream os, IEventTable et){
        List<Set<Integer> > fsmToEvents = new ArrayList<Set<Integer> >(getDimension());
        for(int i = 0 ; i < getDimension() ; i++){
            fsmToEvents.add(new HashSet<Integer>());
        }
        
        for (Map.Entry<Integer,Integer> e: eventToFsm.entrySet()){
            fsmToEvents.get(e.getValue()).add(e.getKey());
        }
        
        for (int i=0;i<fsms.length;i++){
            os.println("FSM_"+i);
            fsms[i].dump(os, et);
            os.println("sync{");
            dumpEventList(os,et,fsmToEvents.get(i));
            os.println("}");
        }
        
        os.println("unbound{");
        dumpEventList(os,et,unbound);
        os.println("}");
    }

}
