/* $Id$
 *
 * dChecker - Distributed Behavior Protocols Checker 
 * Copyright (C) 2006-2007  DSRG, Charles University in Prague
 *                          http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */

package org.ow2.dsrg.fm.dchecker.rmi;



import java.rmi.Remote;
import java.rmi.RemoteException;
import org.ow2.dsrg.fm.bptools.Preferences;


/**
 * Remote interface of clients.
 * 
 * @author thorm
 */
    

public interface Client extends Remote {
    
    /**
     * The client accepts the task from the server, creates an object for this task
     * (implementing ClientTask interface) and the rest of comunication is performed
     * through Task interfaces
     * 
     * @param serverTask RMI stub of the class representing the task on the server
     * @throws java.rmi.RemoteException 
     * @return newly created object that represents the task on the client side.
     */
    ClientTask setTask(ServerTask serverTask,Preferences.Splitting splittingType) throws RemoteException;
    
    /**
     * Invoked by the server to finish the current task
     * @throws java.rmi.RemoteException 
     */
    void taskFinished() throws RemoteException;
    
    void ping() throws RemoteException;
}
