/* $Id$
 *
 * dChecker - Distributed Behavior Protocols Checker 
 * Copyright (C) 2006-2007  DSRG, Charles University in Prague
 *                          http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */

package org.ow2.dsrg.fm.dchecker.rmi;


import org.ow2.dsrg.fm.dchecker.distributed.Area;
import org.ow2.dsrg.fm.dchecker.fly.FlyState;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.util.List;


/**
 * Remote interface of ClientTask.
 * Instance implementing this interface is created for each processed task
 *
 * The instance manages areas in the state space of the task assigned to the 
 * particular client.
 *
 * @author thorm
 */
public interface ClientTask extends Remote {
    
    /**
     * Add area of state space. The area must contain at least one stack containing
     * entry point
     * @param a a new area
     * @throws java.rmi.RemoteException remote exception
     */
    void addArea(Area a) throws RemoteException;
    
    /**
     * Divide one of the assigned areas and return the second half.
     * The area is divided in such way that the returned part does not contain
     * any states already visited.
     *
     * @throws java.rmi.RemoteException remote exception
     * @return part of one of assigned areas
     */
    Area getSubtask() throws RemoteException;
    
    List<FlyState> getStackTrace(FlyState last) throws RemoteException;
    
    /**
     * Divide the assigned area that contains given state and return the second
     * part. The second part will be assigned to another machine.
     *
     * @param s state
     * @throws java.rmi.RemoteException remote exception
     * @return part of assigned area identified by given state
     */
    // Area getSubtask(FlyState s) throws RemoteException;
    
    /**
     * Called when the traversal is finished
     * @throws java.rmi.RemoteException remote exception
     * @return number of visited states.
     
    TerminationInfo terminationDetection() throws RemoteException;
    */
    
    
    void updateAreaMap(int[] areaMember,int clientId) throws RemoteException;
    
    long finish() throws RemoteException;
    
    public void pause() throws RemoteException;
    public void resume()throws RemoteException;
    
    
}
