/* $Id$
 *
 * dChecker - Distributed Behavior Protocols Checker 
 * Copyright (C) 2006-2007  DSRG, Charles University in Prague
 *                          http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */

package org.ow2.dsrg.fm.dchecker.rmi;



import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.rmi.Naming;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.logging.Logger;
import org.ow2.dsrg.fm.bptools.Preferences;



/**
 * Implementation of Client remote interface
 *
 * @author thorm
 */
public class RMIClientImpl extends UnicastRemoteObject implements Client {
    
    /**
     * Creates a new instance of ClientImpl
     * Lookup server in rmi registry
     * @param serverAddress address of the server
     * @throws java.lang.Exception
     */
    public RMIClientImpl(String serverAddress, int p4l) throws Exception {
        super();
        port4listenning = p4l;
        server = (Server) Naming.lookup("//" + serverAddress + "/dcheckerServer");
        
    }
    
    
    
    public int usedPort;
    private int port4listenning;
    
    
    public int getPortForListenning(){
        usedPort = port4listenning;
        return port4listenning;
    }
    
    
    /**
     * This method is run by the main thread. It registers the client,waits until
     * the task is assigned and then runs the task.
     * When the task is finished it waits for another task.
     *
     * @throws java.lang.Exception remote exception
     */
    
    public synchronized void  run() throws Exception {
        InetSocketAddress isa = new InetSocketAddress(InetAddress.getLocalHost(),getPortForListenning());
        clientId = server.registerClient((Client) this, isa);
        
        while (!end) {
            while (task == null) {
                logger().info("Client : Waiting for task");
                try {
                    wait();
                }catch (InterruptedException ie){}
            }
            logger().info("Client : Task assigned");
            task.init();
            task.run();
            task.finalCleanup();
            logger().info("Client : Task finished");
            task = null;
        }
        
    }
    
    
    /**
     * Find proper logger
     * @return the logger
     */
    private Logger logger() {
        return Logger.getLogger("dchecker.Client_" + Integer.toString(clientId));
    }
    
    
    /**
     * The server that manages this client
     */
    private Server server;
    
    /**
     */
    boolean end;
    
    /**
     * Client identifier assigned to this client
     */
    int clientId = -1;
    
    
    /*********************************
     *  Methods Invoked through RMI  *
     *********************************/
    
    /**
     * The client accepts the task from the server, creates an object for this task
     * (implementing ClientTask interface) and the rest of comunication is performed
     * through task interfaces
     *
     * @param serverTask RMI stub of the class representing the task on the server
     * @throws java.rmi.RemoteException
     * @return newly created object that represents the task on the client side.
     */
    public synchronized ClientTask setTask(ServerTask serverTask,Preferences.Splitting splittingType) throws RemoteException {
        assert task == null;
        
        if (splittingType.equals(Preferences.Splitting.Hash)) {
            task = new RMIClientTaskImplHash(this, serverTask);
        } else if (splittingType.equals(Preferences.Splitting.Dynamic)) {
            task = new RMIClientTaskImplDynamic(this, serverTask);
        } else{
            task = new RMIClientTaskImplStatic(this, serverTask);
        }
        
        notifyAll();
        return task;
    }
    
    public void ping() throws RemoteException {}
    
    /**
     * Invoked by the server to finish the current task
     */
    public void taskFinished() {
        task = null;
    }
    
    
    /***********************************
     *  Members used from RMI threads  *
     ***********************************/
    
    
    /**
     * Assigned task
     */
    RMIClientTaskImpl task;
    
}
