/* $Id$
 *
 * dChecker - Distributed Behavior Protocols Checker 
 * Copyright (C) 2006-2007  DSRG, Charles University in Prague
 *                          http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */

package org.ow2.dsrg.fm.dchecker.rmi;


import org.ow2.dsrg.fm.dchecker.distributed.Area;
import org.ow2.dsrg.fm.dchecker.errors.CompositionException;
import org.ow2.dsrg.fm.dchecker.fly.FlyState;
import java.io.IOException;
import java.rmi.NoSuchObjectException;
import java.rmi.RemoteException;
import java.util.List;


/**
 *
 * @author thorm
 */
public class RMIClientTaskImplStatic extends RMIClientTaskImpl {
    
    public RMIClientTaskImplStatic(RMIClientImpl client, ServerTask serverTask) throws RemoteException {
        super(client, serverTask);
    }
    
    public boolean enqueNewEntryPoint(FlyState ep){
        area.newEntryPoint(ep);
        return true;
    }
    
    public boolean visitNStates(int n) throws CompositionException, MachineLostException{
        boolean workDone = false;
        if (area==null) return workDone;
        for (int i = 0; i< n ; i++){
            if (area.traversal(this)){
                workDone = true;
            }else{
                break;
            }
        }
        return workDone;
    }
    
    @Override
    public void finalCleanup() throws  NoSuchObjectException, IOException{
        super.finalCleanup();
        if (area==null) return;
        area.finishUserHandler();
    }
    
    public void outOfArea(FlyState fs, int[] splitted) throws ClientLostException {
        int trg = cid.initialDI.get(splitted);
        stateOutputQueue.sendState(trg,fs);
    }
    
    protected long visited() {
        if (area==null) return 0l;
        return area.getNumberOfVisited();
    }
    
    public void updateAreaMap(int[] areaMember,int clientId) throws RemoteException{
        assert false;
    }
    
    Area area = null;
    
    /***********************************
     *  Methods Invoked through RMI    *
     ***********************************/

    /**
     * Add a new area. This should be invoked only if the station is iddle
     * @param a
     * @throws java.rmi.RemoteException
     */
    public synchronized void addArea(Area a) throws RemoteException {
        assert area == null;
        area = a;
        
        initArea(area);
        
        notifyAll();
    }

    public List<FlyState> getStackTrace(FlyState last) throws RemoteException{
        return area.getStackTrace(last);
    }
    
    public synchronized Area getSubtask() throws RemoteException {
        assert false : "Property splitting is not the same on server and client";
        return null;
    }

    public void processNewAreas() throws RemoteException, ClientLostException
    {
    }
       
    protected boolean noWork(){
        return !stateInputQueue.nextStateAvailable();
    }
    
    
    /***********************************
     *  Members used from RMI threads  *
     ***********************************/

}
