/* $Id$
 *
 * dChecker - Distributed Behavior Protocols Checker 
 * Copyright (C) 2006-2007  DSRG, Charles University in Prague
 *                          http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */

package org.ow2.dsrg.fm.dchecker.rmi;


import org.ow2.dsrg.fm.dchecker.distributed.Area;
import org.ow2.dsrg.fm.dchecker.distributed.BFSArea;
import org.ow2.dsrg.fm.dchecker.distributed.spacemap.BoundingBox;
import org.ow2.dsrg.fm.dchecker.distributed.spacemap.DistributionInfo;
import org.ow2.dsrg.fm.dchecker.fly.FlyFSM;
import org.ow2.dsrg.fm.dchecker.fly.FlyState;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;
import org.ow2.dsrg.fm.bptools.Preferences;
import org.ow2.dsrg.fm.bptools.architecture.ParsedArchitecture;


/**
 *
 * @author thorm
 */
public class RMIServerTaskImplDynamic extends RMIServerTaskImpl {
    
    public RMIServerTaskImplDynamic(RMIServerImpl server, UserInterface ui, ParsedArchitecture arch) throws Exception {
        
        super(server, ui, arch);
        splittingType = Preferences.Splitting.Dynamic;
        reachedAreasToAssign = new LinkedList<Area>();
        newReachedStatesFromNotAssignedAreas = new LinkedList<FlyState>();
        Area initialArea = areas.get(0);
        reachedAreasToAssign.add(initialArea);
    }
    
    
    
    
    
    protected List<Area> createInitialAreas(int numberOfClients){
        
        List<Area> ret = new ArrayList<Area>();
        
        FlyFSM ffsm = architecture.getFlyfsm();
        int dimension = ffsm.getDimension();
        BoundingBox bb = new BoundingBox(dimension);
        notAssignedMap = new DistributionInfo<Area>(dimension);
        
        
        ffsm.fillBoundingBox(bb);
        
        int[] fsmSizes = bb.getMaximalCoord();
        int[] dimsBySizes = sortDimsBySize(fsmSizes);
        
        
        LinkedList<BoundingBox> smallerBoxes = new LinkedList<BoundingBox>();
        LinkedList<BoundingBox> smallBoxes = new LinkedList<BoundingBox>();
        LinkedList<BoundingBox> tmp;
        smallBoxes.add(bb);
        
        int partsPerMachine = 2;
        for(int i = dimsBySizes.length-1;i>=dimsBySizes.length-2;i--){
            int axis = dimsBySizes[i];
            if (fsmSizes[axis] < 1000) break;
            for(BoundingBox smallBox:smallBoxes){
                for(BoundingBox smallerBox:smallBox.split(axis,numberOfClients*partsPerMachine)){
                    smallerBoxes.add(smallerBox);
                }
            }
            partsPerMachine = 1;
            smallBoxes.clear();
            tmp  = smallBoxes;
            smallBoxes = smallerBoxes;
            smallerBoxes = tmp;
        }
        
        
//        FlyFSM ffsm = architecture.getFlyfsm();
        FlyState initState = new FlyState();
        ffsm.init(initState);
        int[] initialStateSplitted = new int[dimension];
        ffsm.getStateParts(initState, initialStateSplitted);

	boolean initialAreaIdentified = false;
        for(int i = 0; i<smallBoxes.size();i++){
            BoundingBox sb = smallBoxes.get(i);
            Area a = new BFSArea(sb);
            
            if (a.belongs(initialStateSplitted)) {
                a.newEntryPoint(initState);
                ret.add(0,a);
		initialAreaIdentified = true;
            }else{
                notAssignedMap.addInterval(a,a);
                ret.add(a);
            }
        }
        
	assert initialAreaIdentified;

        return ret;
    }
    
    protected DistributionInfo<Integer> createInitialDistributionInfo(Set<Integer> clientIds ){
        //return new DistributionInfo<Integer>(architecture.getFlyfsm().getDimension());
        
        
        DistributionInfo<Integer> ret = new DistributionInfo<Integer>(architecture.getFlyfsm().getDimension());
        for (Area a: areas ){
            ret.addInterval(a,null);
        }
         
        return ret;
        
        
    }
    
    
    /**
     * Pick some client and get part of the work assigned to it back so that
     * this part can be assigned to someone else.
     *
     * @throws java.lang.RemoteException
     * @return new area to be assigned
     */
    protected Area getSomeWork(int targetClientId) {
        Area a = reachedAreasToAssign.poll();
        if (a==null) return null;
        notAssignedMap.addInterval(a,null);
        return a;
    }
    
    
    public boolean shouldBeIddle(){
        synchronized (wakeUpper){
        return super.shouldBeIddle() && newReachedStatesFromNotAssignedAreas.isEmpty();
        }
    }
    
    
    public boolean processEvents() throws ClientLostException{
        boolean ret = processNewlyIddleStations();
        processReachedArea();
        return ret;
    }
    
    protected void processReachedArea() throws ClientLostException{
        
        boolean newReachedArea = false;
        FlyFSM ffsm = architecture.getFlyfsm();
        int dimension = ffsm.getDimension();
        int[] parts = new int[dimension];
        
        while(true){
            
            FlyState fs;
            synchronized(this){
                synchronized (wakeUpper){
                fs = newReachedStatesFromNotAssignedAreas.poll();
                }
            }
            if (fs == null ) break;
            
            architecture.getFlyfsm().getStateParts(fs,parts);
            Area reachedArea = notAssignedMap.get(parts);
            if (reachedArea!=null){
                reachedArea.newEntryPoint(fs);
                reachedAreasToAssign.add(reachedArea);
                notAssignedMap.changeValueForWholeArea(parts,null);
                newReachedArea = !iddleStations.isEmpty();
            }
        }
        
        if (newReachedArea){
            int minId=0,minVisited=0;
            for (Integer clientId :iddleStations){
                TerminationInfo ti = terminationDetectionMap.get(clientId);
                if (ti==null) continue;
                long visited = ti.statesVisited;
                if (visited<= minVisited){
                    visited= minVisited;
                    minId = clientId;
                }
            }
            
            Area a = getSomeWork(minId);
            if (a != null) {                
                assignArea(minId,a);
                iddleStations.remove(minId);
            }
        }
        newReachedArea = false;
    }
    
    
    protected long terminationDetection() throws ClientLostException{
        if (!reachedAreasToAssign.isEmpty()) return -1;
        return super.terminationDetection();
    }
    
    
    
    
    private LinkedList<Area> reachedAreasToAssign;
    private DistributionInfo<Area> notAssignedMap;
    
    
    
    /***********************************
     *  Methods Invoked through RMI    *
     ***********************************/
    
    public void newAreaReached(FlyState fs) throws RemoteException{
        synchronized (wakeUpper){
            newReachedStatesFromNotAssignedAreas.add(fs);
            wakeUpper.notifyAll();
        }
    }
    
    /***********************************
     *  Members used from RMI threads  *
     ***********************************/
    
    private LinkedList<FlyState> newReachedStatesFromNotAssignedAreas;
}
