/* $Id$
 *
 * dChecker - Distributed Behavior Protocols Checker 
 * Copyright (C) 2006-2007  DSRG, Charles University in Prague
 *                          http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.ow2.dsrg.fm.dchecker.rmi;

import org.ow2.dsrg.fm.dchecker.distributed.Area;
import org.ow2.dsrg.fm.dchecker.distributed.HashArea;
import org.ow2.dsrg.fm.dchecker.distributed.spacemap.BoundingBox;
import org.ow2.dsrg.fm.dchecker.distributed.spacemap.DistributionInfo;
import org.ow2.dsrg.fm.dchecker.fly.FlyFSM;
import org.ow2.dsrg.fm.dchecker.fly.FlyState;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import org.ow2.dsrg.fm.bptools.Preferences;
import org.ow2.dsrg.fm.bptools.architecture.ParsedArchitecture;

/**
 *
 * @author tom
 */
public class RMIServerTaskImplHash extends RMIServerTaskImplStatic {

    /** Creates a new instance of RMIServerTaskImplHash */
    public RMIServerTaskImplHash(RMIServerImpl server, UserInterface ui, ParsedArchitecture arch) throws Exception {
        super(server, ui, arch);
    }

    {
        splittingType = Preferences.Splitting.Hash;
    }

    @Override
    protected List<Area> createInitialAreas(int numberOfClients) {
        List<Area> areasToAssign = new ArrayList<Area>();
        int dimension = architecture.getFlyfsm().getDimension();
        BoundingBox bb = new BoundingBox(dimension);

        FlyFSM ffsm = architecture.getFlyfsm();
        ffsm.fillBoundingBox(bb);

        int[] clientIdMap = new int[numberOfClients];
        int idx = 0;
        for (int clientId : clientRep.getClientIds()) {
            clientIdMap[idx] = clientId;

            Area a = new HashArea(idx, numberOfClients, clientIdMap, bb);
            a.setFlyFSM(ffsm);

            areasToAssign.add(a);
            idx++;
        }

        FlyState initState = new FlyState();
        architecture.getFlyfsm().init(initState);
        areasToAssign.get(0).newEntryPoint(initState);

        return areasToAssign;
    }

    @Override
    public boolean setAllClientsTask() throws ClientLostException, IOException {
        boolean atLeastOne = false;

        for (int i = 0; i < clientRep.getMaxClientId(); i++) {
            atLeastOne = setClientTask(i) || atLeastOne;
        }

        if (atLeastOne) {
            Iterator<Area> areaIt = areas.iterator();
            for (int clientId : clientRep.getClientIds()) {
                Area a;
                if (areaIt.hasNext()) {
                    a = areaIt.next();
                } else {
                    break;
                }
                clientRep.addArea(clientId, a);
            }
        }
        return atLeastOne;
    }

    @Override
    protected DistributionInfo<Integer> createInitialDistributionInfo(Set<Integer> clientIds) {
        return null;
    }
    /***********************************
     *  Methods Invoked through RMI    *
     ***********************************/
    /***********************************
     *  Members used from RMI threads  *
     ***********************************/
}
