/* $Id$
 *
 * dChecker - Distributed Behavior Protocols Checker 
 * Copyright (C) 2006-2007  DSRG, Charles University in Prague
 *                          http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.ow2.dsrg.fm.dchecker.rmi;

import org.ow2.dsrg.fm.dchecker.distributed.Area;
import org.ow2.dsrg.fm.dchecker.distributed.BFSArea;
import org.ow2.dsrg.fm.dchecker.distributed.spacemap.BoundingBox;
import org.ow2.dsrg.fm.dchecker.distributed.spacemap.DistributionInfo;
import org.ow2.dsrg.fm.dchecker.fly.FlyFSM;
import org.ow2.dsrg.fm.dchecker.fly.FlyState;
import java.io.PrintStream;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import org.ow2.dsrg.fm.bptools.Preferences;
import org.ow2.dsrg.fm.bptools.architecture.ParsedArchitecture;

/**
 *
 * @author thorm
 */
public class RMIServerTaskImplStatic extends RMIServerTaskImpl {

    public RMIServerTaskImplStatic(RMIServerImpl server, UserInterface ui, ParsedArchitecture arch) throws Exception {
        super(server, ui, arch);

    }

    {
        splittingType = Preferences.Splitting.Static;
    }

    protected List<Area> createInitialAreas(int numberOfClients) {
        List<Area> areasToAssign = new ArrayList<Area>();

        int dimension = architecture.getFlyfsm().getDimension();
        BoundingBox bb = new BoundingBox(dimension);

        FlyFSM ffsm = architecture.getFlyfsm();



        ffsm.fillBoundingBox(bb);


        int[] fsmSizes = bb.getMaximalCoord();
        int maxindex = indexOfMaximum(fsmSizes);

        BoundingBox[] smallboxes = bb.split(maxindex, numberOfClients);
        PrintStream ps;



        FlyState initState = new FlyState();
        architecture.getFlyfsm().init(initState);


        for (int i = smallboxes.length - 1; i >= 0; i--) {
            BoundingBox sb = smallboxes[i];
            Area a = new BFSArea(smallboxes[i]);

            FlyFSM ffsmSlice = architecture.getFlyFSMSlice(sb.getMinimalCoord(), sb.getMaximalCoord());
            a.setFlyFSM(ffsmSlice);

            if (a.belongs(initState)) {
                a.newEntryPoint(initState);
            }

            areasToAssign.add(a);
        }


        return areasToAssign;
    }

    protected DistributionInfo<Integer> createInitialDistributionInfo(Set<Integer> clientIds) {
        DistributionInfo<Integer> ret = new DistributionInfo<Integer>(architecture.getFlyfsm().getDimension());
        int areaCnt = 0;
        Iterator<Integer> clientIdIt = clientIds.iterator();
        for (areaCnt = 0; areaCnt < areas.size(); areaCnt++) {
            if (!clientIdIt.hasNext()) {
                break;
            }
            ret.addInterval(areas.get(areaCnt), clientIdIt.next());
        }
        return ret;
    }

    /**
     * Pick some client and get part of the work assigned to it back so that
     * this part can be assigned to someone else.
     *
     * @throws java.lang.RemoteException
     * @return new area to be assigned
     */
    protected Area getSomeWork(int targetClientId) {
        return null;
    }

    /***********************************
     *  Methods Invoked through RMI    *
     ***********************************/
    public void newAreaReached(FlyState fs) throws RemoteException {
        assert false;
    }
    /***********************************
     *  Members used from RMI threads  *
     ***********************************/
}
