/* $Id$
 *
 * dChecker - Distributed Behavior Protocols Checker 
 * Copyright (C) 2006-2007  DSRG, Charles University in Prague
 *                          http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */

package org.ow2.dsrg.fm.dchecker.rmi;


import java.rmi.Naming;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import org.ow2.dsrg.fm.bptools.architecture.BPParserBridge;
import org.ow2.dsrg.fm.bptools.architecture.ParsedArchitecture;


/**
 * Implementation of the user interface that is used by user to enter the tasks
 * to the server
 *
 * @author thorm
 */
public class RMIUserInterfaceImpl extends UnicastRemoteObject implements UserInterface {
    
    /**
     * Creates a new instance of RMIUserInterfaceImpl 
     * @throws java.rmi.RemoteException 
     */
    public RMIUserInterfaceImpl() throws RemoteException {
        super(); 
    }
    
    /**
     * Lookup the server, send him a task and wait for result
     * @param serverAddress address of the server
     * @param protocol textual architecture description
     * @throws java.lang.Exception 
     */
    public synchronized void run(String serverAddress, String fileName) throws Exception {
        Server server = (Server) Naming.lookup(
                "//" + serverAddress + "/dcheckerServer");
        
        ParsedArchitecture pa = ParsedArchitecture.loadArchitecture(fileName, new BPParserBridge());
        server.newTask(this, pa);
        
        while (!end) {
            try {
                wait();
            } catch (InterruptedException e) {}
        }
        
        while (!unexportObject(this, false)) {
            ;
        }
    }
    

    /**
     * The end was notified
     */
    private boolean end = false;
    
    /***********************************
     *  Methods Invoked through RMI    *
     ***********************************/

        /**
     * Used by server to report the progress
     * @param m message to display
     * @throws java.rmi.RemoteException 
     */
    public void msg(String m) throws RemoteException {
        System.out.println(m);
    }
    
    /**
     * After invocation of this method no other message will come from the server
     * The console can be finished.
     *
     * @throws java.rmi.RemoteException 
     */
    public synchronized void finish() throws RemoteException {
        end = true;
        notifyAll();
    }
    
    public void ping() throws RemoteException {}
    
    
    /***********************************
     *  Members used from RMI threads  *
     ***********************************/

}
